      subroutine diag2(m,n,d,x)
c----------------------------------------------------------------------
      implicit double precision (a-h,o-z)
      parameter (maxdim=1000)
      parameter (eps=2.5d-16,dinf=2.3d-308,tol=dinf/eps)
c
c      computation of all eigenvalues and eigenvectors of a real
c      symmetric matrix by the method of qr transformations.
c      if the euclidean norm of the rows varies   s t r o n g l y
c      most accurate results may be obtained by permuting rows and
c      columns to give an arrangement with increasing norms of rows.
c
c      two machine constants must be adjusted appropriately,
c      eps = minimum of all x such that 1+x is greater than 1 on the
c            computer,
c      tol = inf / eps  with inf = minimum of all positive x represen-
c            table within the computer.
c      a dimension statement e(160) may also be changed appropriately.
c
c      input
c
c      (m)   not larger than 160,  corresponding value of the actual
c            dimension statement a(m,m), d(m), x(m,m),
c      (n)   not larger than (m), order of the matrix,
c      (a)   the matrix to be diagonalized, its lower triangle has to
c            be given as  ((a(i,j), j=1,i), i=1,n),
c.....
c.....the matrix #a# has been removed from the procedure
c.....the matrix #x# has to be put up by a predecessor routine c.....
c
c      output
c
c      (d)   components d(1), ..., d(n) hold the computed eigenvalues
c            in ascending sequence. the remaining components of (d) are
c            unchanged,
c      (x)   the computed eigenvector corresponding to the j-th eigen-
c            value is stored as column (x(i,j), i=1,n). the eigenvectors
c            are normalized and orthogonal to working accuracy. the
c            remaining entries of (x) are unchanged.
c
c      array (a) is unaltered. however, the actual parameters
c      corresponding to (a) and (x)  may be identical, ''overwriting''
c      the eigenvectors on (a).
c
c      leibniz-rechenzentrum, munich 1965
c
      dimension   d(m), x(m,m)
      dimension   e(maxdim)
c
c     correct adjustment for ibm 360/91 double precision
c
      if(m.gt.maxdim) then
        write(6,10) m,maxdim
10      format(' DIMENSION TOO LARGE IN DIAG2:',2i8)
        stop 'error on diag2'
      end if
c     call accnt('diag2',1)
c     eps=dlamch('e')
c     tol=dlamch('u')/eps
c
      if(n.eq.1) go to 400
      do 11 i=1,n
      d(i)=0
11    e(i)=0
c
c     householder's reduction
c
      do 150 i=n,2,-1
      l=i-2
      h=0.0d0
      g=x(i,i-1)
      if(l.le.0) goto 140
      do 30 k=1,l
   30 h=h+x(i,k)*x(i,k)
      s=h+g*g
      if(s.lt.tol) then
        h=0.0d0
      else if(h.gt.0) then
        l=l+1
        f=g
        g=dsqrt(s)
        if(f.gt.0) g=-g
        h=s-f*g
        x(i,i-1)=f-g
        f=0.0d0
c
        do 110 j=1,l
        x(j,i)=x(i,j)/h
        s=0.0d0
        do 80 k=1,j
   80   s=s+x(j,k)*x(i,k)
        j1=j+1
        if(j1.gt.l) go to 100
        do 90 k=j1,l
   90   s=s+x(k,j)*x(i,k)
  100   e(j)=s/h
  110   f=f+s*x(j,i)
c
        f=f/(2.d0*h)
c
        do 120 j=1,l
  120   e(j)=e(j)-f*x(i,j)
c
        do 130 j=1,l
        f=x(i,j)
        s=e(j)
        do 130 k=1,j
  130   x(j,k)=x(j,k)-f*e(k)-x(i,k)*s
c
      end if
  140 d(i)=h
  150 e(i-1)=g
c
c     accumulation of transformation matrices
c
  160 d(1)=x(1,1)
      x(1,1)=1.0d0
      do 220 i=2,n
      l=i-1
      if(d(i)) 200,200,170
  170 do 190 j=1,l
      s=0.0d0
      do 180 k=1,l
  180 s=s+x(i,k)*x(k,j)
      do 190 k=1,l
  190 x(k,j)=x(k,j)-s*x(k,i)
  200 d(i)=x(i,i)
      x(i,i)=1.0d0
  210 do 220 j=1,l
      x(i,j)=0.0d0
  220 x(j,i)=0.0d0
c
c     diagonalization of the tridiagonal matrix
c
      b=0.0
      f=0.0
      e(n)=0.0d0
c
      do 340 l=1,n
      h=eps*(dabs(d(l))+dabs(e(l)))
      if (h.gt.b) b=h
c
c     test for splitting
c
      do 240 j=l,n
      if (dabs(e(j)).le.b) goto 250
  240 continue
c
c     test for convergence
c
  250 if(j.eq.l) go to 340
c
c     shift from upper 2*2 minor
c
  260 p=(d(l+1)-d(l))*0.5d0
      r=dsqrt(p*p+e(l)*e(l))
      if(p.lt.0) then
        p=p+r
      else
        p=p-r
      end if
  290 h=d(l)+p
      do 300 i=l,n
  300 d(i)=d(i)-h
      f=f+h
c
c     qr transformation
c
      p=d(j)
      c=1.0d0
      s=0.0d0
c
c     simulation of loop do 330 i=j-1,l,(-1)
c
      j1=j-1
      do 330 i=j1,l,-1
      g=c*e(i)
      h=c*p
c
c     protection against underflow of exponents
c
      if (dabs(p).lt.dabs(e(i))) goto 310
      c=e(i)/p
      r=dsqrt(c*c+1.0d0)
      e(i+1)=s*p*r
      s=c/r
      c=1.0d0/r
      go to 320
  310 c=p/e(i)
      r=dsqrt(c*c+1.0d0)
      e(i+1)=s*e(i)*r
      s=1.0d0/r
      c=c/r
  320 p=c*d(i)-s*g
      d(i+1)=h+s*(c*g+s*d(i))
      do 330 k=1,n
      h=x(k,i+1)
      x(k,i+1)=x(k,i)*s+h*c
  330 x(k,i)=x(k,i)*c-h*s
c
      e(l)=s*p
      d(l)=c*p
      if (dabs(e(l)).gt.b) go to 260
c
c     convergence
c
  340 d(l)=d(l)+f
c
c     ordering of eigenvalues
c
      ni=n-1
  350 do 380i=1,ni
      k=i
      p=d(i)
      j1=i+1
      do 360j=j1,n
      if(d(j).ge.p) goto 360
      k=j
      p=d(j)
  360 continue
      if (k.eq.i) goto 380
      d(k) =d(i)
      d(i)=p
      do 370 j=1,n
      p=x(j,i)
      x(j,i)=x(j,k)
  370 x(j,k)=p
  380 continue
c
c     fixing of sign
c
      do 385 i=1,n
      pm=0
      do 386 j=1,n
      if(pm.gt.dabs(x(j,i))) goto 386
      pm =dabs(x(j,i))
      k=j
  386 continue
      if(x(k,i).ge.0) goto 385
      do 387 j=1,n
  387 x(j,i)=-x(j,i)
  385 continue
  390 go to 410
c
c     special treatment of case n = 1
c
  400 d(1)=x(1,1)
      x(1,1)=1.0d0
  410 continue
c     call accnt(' ',2)
      return
      end
